﻿using System.ComponentModel;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace SharpObservation.Tests
{

    using Dummy;

    [TestClass]
    public class ObservabeObjectTests
    {
        /// <summary>
        /// Tests the property changing invoked correctly.
        /// </summary>
        [TestMethod]
        public void TestPropertyChangingInvokedCorrectly()
        {
            var dummy = new NotifyDummy{ Name = "John" };

            Assert.AreEqual("John", dummy.Name);

            dummy.PropertyChanging +=
                (o, e) =>
                {
                    var args = e as PropertyChangingEventArgs<string>;
                    Assert.AreSame(o, dummy);
                    Assert.AreEqual("Name", e.PropertyName);
                    Assert.IsNotNull(args);
                    Assert.AreEqual("John", args.Current);
                    Assert.AreEqual("Jane", args.Proposed);
                };


            dummy.Name = "Jane";

        }



        /// <summary>
        /// Tests the property changed invoked correctly.
        /// </summary>
        [TestMethod]
        public void TestPropertyChangedInvokedCorrectly()
        {
            var dummy = new NotifyDummy{ Name = "John" };

            Assert.AreEqual("John", dummy.Name);

            dummy.PropertyChanged +=
                (o, e) =>
                {
                    var args = e as PropertyChangedEventArgs<string>;
                    Assert.AreSame(o, dummy);
                    Assert.AreEqual("Name", e.PropertyName);
                    Assert.IsNotNull(args);
                    Assert.AreEqual("John", args.Previous);
                    Assert.AreEqual("Jane", args.Current);
                };

            dummy.Name = "Jane";
        }



        /// <summary>
        /// Tests the is notification active property.
        /// </summary>
        [TestMethod]
        public void TestIsNotificationActive()
        {
            var dummy = new NotifyDummy() { Name = "John" };
            Assert.IsFalse(dummy.IsNotificationActive);

            var myEvent = new PropertyChangedEventHandler((o, e) => { });
            dummy.PropertyChanged += myEvent;

            Assert.IsTrue(dummy.IsNotificationActive);

            dummy.PropertyChanged -= myEvent;

            Assert.IsFalse(dummy.IsNotificationActive);

        }
    }
}
