﻿#region

using System.Collections.Generic;
using System.Linq;
using System.Threading;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using SharpObservation.Tests.Dummy;
using SharpObservation.Virtual;

#endregion

namespace SharpObservation.Tests
{
    ///<summary>
    ///  This is a test class for VirtualCollectionTest and is intended
    ///  to contain all VirtualCollectionTest Unit Tests
    ///</summary>
    [TestClass]
    public class VirtualCollectionTests
    {
        public TestContext TestContext { get; set; }

        [TestMethod]
        public void TestAsyncCountRequestedAutomatically()
        {
            var tester = new TestInfo(true);     // async, no delays
            var collection = tester.Collection;

            var assumed = collection.AssumedCount = 12345;
            var count = collection.Count;

            Assert.AreEqual(LoadingState.Loading, collection.CountState);
            Assert.AreEqual(assumed, count);

            while (collection.CountState == LoadingState.Loading)
                Thread.Sleep(100);

            count = collection.Count;

            Assert.AreEqual(LoadingState.Loaded, collection.CountState);
            Assert.AreNotEqual(assumed, count);
            Assert.AreEqual(tester.Provider.Count, count);
        }


        [TestMethod]
        public void TestCountSynchronous()
        {
            var tester = new TestInfo(false);     // sync, no delays
            var collection = tester.Collection;

            var assumed = collection.AssumedCount = 12345;
            var count = collection.Count;

            Assert.AreEqual(LoadingState.Loaded, collection.CountState);
            Assert.AreNotEqual(assumed, count);
        }


        [TestMethod]
        public void VirtualCollectionGetsItemsSynchronously()
        {
            const int pageSize = 10;
            var tester = new TestInfo(false, pageSize);     // async, delays, small pages
            var collection = tester.Collection;
            var providerCount = collection.AssumedCount = tester.Provider.Count;

            var references = new List<ItemReference<DummyDataItem>>(providerCount);

            for (int i = 0; i < providerCount; i++)
                references.Add(collection[i]);

            // all items should be dead
            Assert.IsTrue(references.All(c => c.IsAlive));
        }

        [TestMethod]
        public void VirtualCollectionGetsItemsAsynchronously()
        {
            const int pageSize = 10;
            var tester = new TestInfo(true, pageSize);     // async, delays, small pages
            var collection = tester.Collection;
            var providerCount = collection.AssumedCount = tester.Provider.Count;

            var references = new List<ItemReference<DummyDataItem>>(providerCount);

            for (int i = 0; i < providerCount; i++)
                references.Add(collection[i]);
        
            while (references.Any(i => i.Page.State != LoadingState.Loaded))
                Thread.Sleep(10);
            
            // all items should be alive
            Assert.IsTrue(references.All(c => c.IsAlive));
        }
    }
}