﻿using System;

namespace SharpObservation
{
    /// <summary>
    /// Provides support for Undo infrastructure, as well as rich information about a property change event
    /// </summary>
    /// <typeparam name="TInstance">The type which owns the property which changed</typeparam>
    /// <typeparam name = "TProp">The type of the property that changed</typeparam>
    public class UndoablePropertyChangedEventArgs<TInstance, TProp> 
        : PropertyChangedEventArgs<TProp>
        where TInstance : ObservableObject
    {
        private readonly TInstance _owner;
        private readonly Action<TInstance, TProp> _setter;

        /// <summary>
        ///   Constructs a new instance of UndoablePropertyChangedEventArgs
        /// </summary>
        /// <param name = "propertyName">The name of the changes _property</param>
        /// <param name = "previous">The factory  of the _property</param>
        /// <param name = "current">The factory  of the _property</param>
        /// <param name = "owner">The observable object which contains the property being changed</param>
        /// <param name = "setter"></param>
        public UndoablePropertyChangedEventArgs(
            string propertyName,
            TProp previous,
            TProp current,
            TInstance owner,
            Action<TInstance, TProp> setter)
            : base(propertyName, previous, current)
        {
            _owner = owner;
            _setter = setter;
        }

        /// <summary>
        ///   Undoes a property change so that the property reflects it's former value
        /// </summary>
        public void Undo()
        {
            _owner.DoSetProperty(PropertyName, Current, Previous, _setter);
        }
    }
}