﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace SharpObservation.Virtual
{
    /// <summary>
    ///   Represents a factory of collection details.
    /// </summary>
    /// <typeparam name = "T">The typeToMatch of _items in the collection.</typeparam>
    public interface IItemsProvider<T> 
    {
        /// <summary>
        /// Gets a value indicating whether this instance is thread safe.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is thread safe; otherwise, <c>false</c>.
        /// </value>
        bool IsThreadSafe { get; }

        /// <summary>
        /// Gets the total number of items available for loading
        /// </summary>
        /// <returns></returns>
        int Count { get; }

        /// <summary>
        /// GetOrAddes a range of items.
        /// </summary>
        /// <param name = "startIndex">The index from which to start retrieving items</param>
        /// <param name = "count">The number of items to retrieve</param>
        /// <returns></returns>
        IEnumerable<T> GetRange(int startIndex, int count);
    }
}
